<?php

namespace Drupal\commerce_pado\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_cart\Form\AddToCartForm;
use Drupal\commerce_order\Entity\OrderItem;
use Drupal\commerce\Context;
use Drupal\Core\Link;

/**
 * Provides the order item add to cart form.
 */
class PadoAddToCartForm extends AddToCartForm {

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    // Add add-ons.
    $field_name = $form_state->get(['settings', 'add_on_field']);
    /** @var \Drupal\commerce_product\Entity\ProductInterface $product */
    $product = $form_state->get('product');
    $multiple = $form_state->get(['settings', 'multiple']);
    $options = [];
    $add_ons = [];

    /** @var \Drupal\commerce_product\Entity\ProductInterface $add_on_product */
     foreach ($product->{$field_name} as $add_on_product) {
       foreach ($add_on_product->entity->variations as $add_on_variation_field) {
         /** @var \Drupal\commerce_product\Entity\ProductVariationInterface $add_on_variation */
        $add_on_variation = $add_on_variation_field->entity;
        $add_on_variation_id = $add_on_variation->id();
        $view_builder = $this->entityTypeManager->getViewBuilder('commerce_product_variation');
        $options[$add_on_variation_id] = $add_on_variation->getTitle();
        $add_ons[$add_on_variation_id]['#description'] = $view_builder->view($add_on_variation, 'add_on');
      }
    }
    if (!empty($options)) {
      if (!$multiple) {
        $options = [0 => $this->t('No add-on')] + $options;
      }
      $form['add_ons'] = [
        '#type' => empty($multiple) ? 'radios' : 'checkboxes',
        '#options' => $options,
        '#title' => $this->t('Add-ons'),
      ] + $add_ons;
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $add_ons = $form_state->getValue('add_ons');
    $combine = $form_state->get(['settings', 'combine']);
    if (!is_array($add_ons)) {
      $add_ons = [$add_ons => $add_ons];
    }
    $add_ons = array_filter($add_ons);
    foreach ($add_ons as $add_on_variation_id) {
      /** @var \Drupal\commerce_product\Entity\ProductVariation $add_on_variation */
      $add_on_variation = $this->entityTypeManager->getStorage('commerce_product_variation')->load($add_on_variation_id);

      // @todo Allow providing quantity in the add to cart form.
      $quantity = 1;
      $order_item_properties = [
        'type' => $add_on_variation->getOrderItemTypeId(),
        // The strict comparison in OrderItemMatcher::matchAll() requires
        // us to cast the id to string.
        'purchased_entity' => (string) $add_on_variation_id,
        'quantity' => $quantity,
      ];
      /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
      $order_item = OrderItem::create($order_item_properties);
      $store = $this->selectStore($add_on_variation);
      $context = new Context($this->currentUser, $store);
      $resolved_price = $this->chainPriceResolver->resolve($add_on_variation, $quantity, $context);
      $order_item->setTitle($add_on_variation->getOrderItemTitle());
      $order_item->setUnitPrice($resolved_price);
      //$order_item->save();

      $order_type_id = $this->orderTypeResolver->resolve($order_item);

      $cart = $this->cartProvider->getCart($order_type_id, $store);
      $this->cartManager->addOrderItem($cart, $order_item, $combine);

      drupal_set_message($this->t('@entity added to @cart-link.', [
        '@entity' => $add_on_variation->label(),
        '@cart-link' => Link::createFromRoute($this->t('your cart', [], ['context' => 'cart link']), 'commerce_cart.page')->toString(),
      ]));
    }
  }

}
